@@ -43,8 +43,9 @@ const char *version = VERSION_STRING;	/* Program version */
 
 static pid_t my_pgrp;		        /* The "magic" process group */
 static pid_t my_pid;		        /* The pid of this process */
+static char *pid_file = NULL;	        /* File in which to keep pid */
 
-int kproto_version;		/* Kernel protocol version */
+int kproto_version;		        /* Kernel protocol version used */
 
 static int mounted = 0;
 
@@ -443,13 +444,16 @@ static void sig_child(int sig)
 
 static void become_daemon(void)
 {
+  FILE *pidfp;
   pid_t pid;
   int nullfd;
   
   /* Don't BUSY any directories unneccessarily */
+
   chdir("/");
   
-  /* Detach */
+  /* Detach from foreground process */
+
   pid = fork();
   if ( pid > 0 )
     exit(0);
@@ -459,12 +463,15 @@ static void become_daemon(void)
   }
   
   /* Open syslog */ 
+
   openlog("automount", LOG_PID, LOG_DAEMON);
-  
+
   /* Initialize global data */
+
   my_pid = getpid();
-  
+
   /* Make our own process group for "magic" reason */
+
   if ( setpgrp() ) {
     syslog(LOG_CRIT, "setpgrp: %m");
     exit(1);
@@ -472,7 +479,7 @@ static void become_daemon(void)
   my_pgrp = getpgrp();
   
   /* Redirect all our file descriptors to /dev/null */
-  
+
   if ( (nullfd = open("/dev/null", O_RDWR)) < 0 ) {
     syslog(LOG_CRIT, "cannot open /dev/null: %m");
     exit(1);
@@ -485,6 +492,35 @@ static void become_daemon(void)
     exit(1);
   }
   close(nullfd);
+
+  /* Write pid file if requested */
+
+  if ( pid_file ) {
+    if ( (pidfp = fopen(pid_file, "wt")) ) {
+      fprintf(pidfp, "%lu\n", (unsigned long) my_pid);
+      fclose(pidfp);
+    } else {
+      syslog(LOG_WARNING, "failed to write pid file %s: %m", pid_file);
+      pid_file = NULL;
+    }
+  }
+}
+
+/*
+ * cleanup_exit() is valid to call once we have daemonized
+ */
+
+static void cleanup_exit(int exit_code)
+{
+  if ( ap.lookup )
+    close_lookup(ap.lookup);
+
+  if ( pid_file )
+    unlink(pid_file);
+
+  closelog();
+
+  exit(exit_code);
 }
 
 static unsigned long getnumopt(char *str, char option)
@@ -512,9 +548,10 @@ int main(int argc, char *argv[])
   struct sigaction sa;
   int mapargc, opt;
   static const struct option long_options[] = {
-    {"help",    0, 0, 'h'},
-    {"timeout", 1, 0, 't'},
-    {"version", 0, 0, 'v'},
+    {"help",     0, 0, 'h'},
+    {"pid-file", 1, 0, 'p'},
+    {"timeout",  1, 0, 't'},
+    {"version",  0, 0, 'v'},
     {0,0,0,0}
   };
   
@@ -523,12 +560,15 @@ int main(int argc, char *argv[])
   ap.exp_timeout = DEFAULT_TIMEOUT;
   
   opterr = 0;
-  while ( (opt = getopt_long(argc, argv, "+ht:v", long_options,
+  while ( (opt = getopt_long(argc, argv, "+hp:t:v", long_options,
 			     NULL)) != EOF ) {
     switch( opt ) {
     case 'h':
       usage();
       exit(0);
+    case 'p':
+      pid_file = optarg;
+      break;
     case 't':
       ap.exp_timeout = getnumopt(optarg, opt);
       break;
@@ -556,6 +596,7 @@ int main(int argc, char *argv[])
     exit(1);
   }
   
+  memset(&ap, 0, sizeof ap);	/* Initialize ap so we can test for null */
   become_daemon();
   
   path    = argv[0];
@@ -578,14 +619,26 @@ int main(int argc, char *argv[])
     *(mapfmt++) = '\0';
   
   if ( !(ap.lookup = open_lookup(map, "", mapfmt, mapargc, mapargv)) )
-    exit(1);
+    cleanup_exit(1);
   
   if (mount_autofs(path) < 0) {
     syslog(LOG_CRIT, "%s: mount failed!", path);
-    close_lookup(ap.lookup);
-    exit(1);
+    cleanup_exit(1);
   }	
   
+  sa.sa_handler = SIG_IGN;
+  sigemptyset(&sa.sa_mask);
+  sa.sa_flags = 0;
+#ifdef SIGUNUSED
+  sigaction(SIGUNUSED, &sa, NULL);
+#endif
+#ifdef SIGLOST
+  sigaction(SIGLOST, &sa, NULL);
+#endif
+  sigaction(SIGVTALRM, &sa, NULL);
+  sigaction(SIGPROF, &sa, NULL);
+  sigaction(SIGPIPE, &sa, NULL);
+  
   sa.sa_handler = sig_shutdown;
   sigemptyset(&sa.sa_mask);
   sigaddset(&sa.sa_mask, SIGCHLD);
@@ -599,9 +652,13 @@ int main(int argc, char *argv[])
   sigaction(SIGBUS, &sa, NULL);
   sigaction(SIGFPE, &sa, NULL);
   sigaction(SIGSEGV, &sa, NULL);
-  sigaction(SIGPIPE, &sa, NULL);
   sigaction(SIGTERM, &sa, NULL);
   sigaction(SIGIO, &sa, NULL);
+  sigaction(SIGXCPU, &sa, NULL);
+  sigaction(SIGXFSZ, &sa, NULL);
+#ifdef SIGSYS
+  sigaction(SIGSYS, &sa, NULL);
+#endif
 #ifdef SIGPWR
   sigaction(SIGPWR, &sa, NULL);
 #endif
@@ -609,7 +666,7 @@ int main(int argc, char *argv[])
   sigaction(SIGSTKFLT, &sa, NULL);
 #endif
   sigaction(SIGUSR2, &sa, NULL);
-  
+
   sa.sa_handler = sig_child;
   sigemptyset(&sa.sa_mask);
   sa.sa_flags = 0;
@@ -619,6 +676,7 @@ int main(int argc, char *argv[])
   sigemptyset(&sa.sa_mask);
   sa.sa_flags = 0;
   sigaction(SIGUSR1, &sa, NULL);
+  sigaction(SIGHUP, &sa, NULL);
   
   sa.sa_handler = sig_expire;
   sigemptyset(&sa.sa_mask);
@@ -661,8 +719,6 @@ int main(int argc, char *argv[])
   
   close_lookup(ap.lookup);
   umount_autofs(1);
-  
-  closelog();
-  
-  return 0;
+
+  cleanup_exit(0);
 }
