@@ -33,7 +33,9 @@ struct lookup_mod *open_lookup(char *name, char *err_prefix,
   sprintf(fnbuf, "%s//lookup_%s.so", AUTOFS_LIB_DIR, name);
   
   if ( !(dh = dlopen(fnbuf, RTLD_NOW)) ) {
-    if ( err_prefix ) syslog(LOG_CRIT, "%scannot open lookup module %s", err_prefix, name);
+    if ( err_prefix )
+      syslog(LOG_CRIT, "%scannot open lookup module %s (%s)",
+	     err_prefix, name, dlerror());
     free(mod);
     return NULL;
   }
@@ -97,7 +99,8 @@ struct parse_mod *open_parse(char *name, char *err_prefix,
   
   if ( !(dh = dlopen(fnbuf, RTLD_NOW)) ) {
     if ( err_prefix )
-      syslog(LOG_CRIT, "%scannot open parse module %s", err_prefix, name);
+      syslog(LOG_CRIT, "%scannot open parse module %s (%s)",
+	     err_prefix, name, dlerror());
     free(mod);
     return NULL;
   }
@@ -161,7 +164,8 @@ struct mount_mod *open_mount(char *name, char *err_prefix)
   
   if ( !(dh = dlopen(fnbuf, RTLD_NOW)) ) {
     if ( err_prefix )
-      syslog(LOG_CRIT, "%scannot open mount module %s", err_prefix, name);
+      syslog(LOG_CRIT, "%scannot open mount module %s (%s)",
+	     err_prefix, name, dlerror());
     free(mod);
     return NULL;
   }
