@@ -39,18 +39,27 @@ struct substvar {
 struct parse_context {
   char *optstr;			/* Mount options */
   struct substvar *subst;	/* $-substitutions */
+  int slashify_colons;		/* Change colons to slashes? */
 };
 
 struct utsname un;
 char processor[65];		/* Not defined on Linux, so we make our own */
 
 static struct substvar		/* Predefined variables: tail of link chain */
-  sv_arch   = { "ARCH",   un.machine, NULL },
-  sv_cpu    = { "CPU",    processor,  &sv_arch },
-  sv_host   = { "HOST",   un.machine, &sv_cpu },
-  sv_osname = { "OSNAME", un.sysname, &sv_host },
-  sv_osrel  = { "OSREL",  un.release, &sv_osname },
-  sv_osvers = { "OSVERS", un.version, &sv_osrel };
+  sv_arch   = { "ARCH",   un.machine,  NULL },
+  sv_cpu    = { "CPU",    processor,   &sv_arch },
+  sv_host   = { "HOST",   un.nodename, &sv_cpu },
+  sv_osname = { "OSNAME", un.sysname,  &sv_host },
+  sv_osrel  = { "OSREL",  un.release,  &sv_osname },
+  sv_osvers = { "OSVERS", un.version,  &sv_osrel };
+
+/* Default context pattern */
+
+static struct parse_context default_context = {
+  NULL,				/* No mount options */
+  &sv_osvers,			/* The substvar predefined variables */
+  1				/* Do slashify_colons */
+};
 
 /* Free all storage associated with this context */
 static void kill_context(struct parse_context *ctxt)
@@ -83,14 +92,16 @@ static struct substvar *findvar(struct substvar *sv, char *str, int len)
 
 /* $- and &-expand a Sun-style map entry and return the length of the entry.
    If "dst" is NULL, just count the length. */
-static int expandsunent(char *src, char *dst, char *key, struct substvar *svc)
+static int expandsunent(char *src, char *dst, char *key, struct substvar *svc,
+			int slashify_colons)
 {
   struct substvar *sv;
-  int len, l;
+  int len, l, seen_colons;
   char *p, ch;
 
   len = 0;
-
+  seen_colons = 0;
+  
   while ( (ch = *src++) ) {
     switch ( ch ) {
     case '&':
@@ -147,6 +158,12 @@ static int expandsunent(char *src, char *dst, char *key, struct substvar *svc)
       }
       break;
 
+    case ':':
+      if ( dst ) *(dst++) = (seen_colons && slashify_colons) ? '/' : ':';
+      len++;
+      seen_colons = 1;
+      break;
+
     default:
       if ( dst ) *(dst++) = ch;
       len++;
@@ -205,13 +222,30 @@ int chunklen(char *whence)
   }
 }
 
+/* Compare str with pat.  Return 0 if compare equal or str is an abbreviation of
+   pat of no less than mchr characters. */
+int strmcmp(char *str, char *pat, int mchr)
+{
+  int nchr = 0;
+
+  while ( *str == *pat ) {
+    if ( ! *str ) return 0;
+    str++; pat++; nchr++;
+  }
+
+  if ( ! *str && nchr > mchr )
+    return 0;
+
+  return *pat - *str;
+}
+
 int parse_init(int argc, char **argv, void **context)
 {
   struct parse_context *ctxt;
   struct substvar *sv;
   char *noptstr;
   int optlen, len;
-  int i;
+  int i, bval;
 
   /* Get processor information for predefined escapes */
 
@@ -233,13 +267,11 @@ int parse_init(int argc, char **argv, void **context)
   }
   *context = (void *) ctxt;
   
-  ctxt->subst = &sv_osvers;
+  *ctxt = default_context;
+  optlen = 0;
 
   /* Look for options and capture, and create new defines if we need to */
 
-  ctxt->optstr = NULL;
-  optlen = 0;
-
   for ( i = 0 ; i < argc ; i++ ) {
     if ( argv[i][0] == '-' ) {
       switch(argv[i][1]) {
@@ -272,6 +304,24 @@ int parse_init(int argc, char **argv, void **context)
 	  ctxt->subst = sv;
 	}
 	break;
+
+      case '-':
+	if ( !strncmp(argv[i]+2, "no-", 3) ) {
+	  noptstr = argv[i]+5;
+	  bval = 0;
+	} else {
+	  noptstr = argv[i]+2;
+	  bval = 1;
+	}
+
+	if ( strmcmp(noptstr, "slashify-colons", 1) ) {
+	  ctxt->slashify_colons = bval;
+	}
+	else
+	  syslog(LOG_ERR, MODPREFIX "unknown option: %s", argv[i]);
+
+	break;
+
       default:
 	syslog(LOG_ERR, MODPREFIX "unknown option: %s", argv[i]);
 	break;
@@ -321,13 +371,13 @@ int parse_mount(char *root, char *name, int name_len, char *mapent, void *contex
   int mapent_len, rv;
   int l, optlen;
 
-  mapent_len = expandsunent(mapent,NULL,name,ctxt->subst);
+  mapent_len = expandsunent(mapent,NULL,name,ctxt->subst,ctxt->slashify_colons);
   pmapent = alloca(mapent_len + 1);
   if (!pmapent) {
     syslog(LOG_ERR, MODPREFIX "alloca: %m");
     return 1;
   }
-  expandsunent(mapent,pmapent,name,ctxt->subst);
+  expandsunent(mapent,pmapent,name,ctxt->subst,ctxt->slashify_colons);
 
   syslog(LOG_DEBUG, "expanded entry: %s", pmapent);
 
